#! /usr/bin/perl
# test_links.pl
# Author: Peter Torpey
# www.petertorpey.com
# Copyright (c) 2003 by Peter Torpey.
#
# This code remains the property of the author. This code may be freely
# distributed so long as this statement, copyright, and notice of the
# original author remain intact.
#
# The script takes a standard HTML bookmark file (used by Netscape/Mozilla or
# as exported from IE) on STDIN and writes a similar link file to STDOUT. The
# URL of each bookmark is visited. If no error was encountered, the link is
# colored green in the resulting HTML, otherwise the link appears red. This
# provides a visual cue as to which links might be dead. Since the script only
# checks against the full URL, the domain may still exist or the page may have
# been moved and the browser could be redirected even though the server
# returns an error. So, check the red links by hand, just to be sure. The script
# takes some time to run, on the order of the number of links, since it is not
# threaded or using multiplexed IO.
#
# Usage: ./test_links.pl [-v] [-t timeout] < bookmark.htm > checked.html &
# Options:
#     -v          Toggle verbose mode which writes the success or failure
#                 of each URL on STERR.
#     -t timeout  Sets the timeout to wait for each server response. The
#                 the default value is 8 seconds.

use LWP::Simple qw(get $ua);

$verbose = false;
$tout = 8;

foreach $argn (0 .. $#ARGV) {
	if ($ARGV[$argn] eq "-v") { $verbose = true; }
	if ($ARGV[$argn] eq "-t") { $tout = $ARGV[++$argn] || die "Invalid timeout."; }
}

$ua->timeout($tout);

while (<STDIN>) {
	$line = $_;
	if (/<DT><A\ HREF=\"(.*?)\"(.*?)>(.*?)<\/A>/) {
		if ($verbose) { print STDERR "$3 - "; }
		if (get($1)) {
			print "<DT><A STYLE=\"color: #009900;\" HREF=\"$1\"$2>$3<\/A>\n";
			if ($verbose) { print STDERR "SUCCESS\n"; }
		}
		else {
			print "<DT><A STYLE=\"color: #FF0000;\" HREF=\"$1\"$2>$3<\/A>\n";
			if ($verbose) { print STDERR "FAIL\n"; }
		}
	}
	else {
		print "$line";
	}
}
